package com.hrsk.cloud.op.adapter.common.security;

import com.hrsk.cloud.op.adapter.common.IpUtils;
import com.hrsk.cloud.op.infrastructure.common.Constants;
import com.hrsk.cloud.op.infrastructure.repository.database.redis.RedisCache;
import com.hrsk.cloud.op.infrastructure.repository.database.redis.RedisKeyEnum;
import eu.bitwalker.useragentutils.UserAgent;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @author: bianlanzhou
 * @create: 2024-10-14 14:50
 * @description: http security 工具
 **/
@Component
public class HttpSecurityHelper {

    /**
     * JWT令牌前缀
     */
    public static final String LOGIN_USER_KEY = "login_user_key";

    /**
     * HEADER令牌前缀
     */
    public static final String HEADER_TOKEN_PREFIX = "Bearer ";


    @Value("${security.token.secret}")
    private String secret;

    @Value("${security.token.header}")
    private String header;

    @Value("${security.token.expireTime}")
    private int expireTime;

    @Resource
    private RedisCache redisCache;

    /**
     * 通过token获取当前登录用户
     * @param token token令牌
     * @return 当前登录用户
     */
    public CurrentLoginUser getCurrentLoginUser(String token){
        if(StringUtils.isEmpty(token)){
            return null;
        }
        Claims claims = Jwts.parser()
                .setSigningKey(secret)
                .parseClaimsJws(token)
                .getBody();
        //解析对应的权限以及用户信息
        String uuid = (String) claims.get(LOGIN_USER_KEY);
        String userKey = RedisKeyEnum.LOGIN_TOKEN_KEY.getKey(uuid);
        return redisCache.getCacheObject(userKey);
    }

    /**
     * 通过request获取当前登录用户
     * @param request httprequest
     * @return 当前登录用户
     */
    public CurrentLoginUser getCurrentLoginUser(HttpServletRequest request){
        String token = getToken(request);
        return getCurrentLoginUser(token);
    }

    /**
     * 获取TOKEN
     * @param request request
     * @return token
     */
    public String getToken(HttpServletRequest request){
        String token = request.getHeader(header);
        if (StringUtils.isNotEmpty(token) && token.startsWith(HEADER_TOKEN_PREFIX)) {
            token = token.replace(HEADER_TOKEN_PREFIX, "");
        }
        return token;
    }

    /**
     * 删除登录缓存
     * @param token token令牌
     */
    public void delLoginCache(String token){
        redisCache.deleteObject(RedisKeyEnum.LOGIN_TOKEN_KEY.getKey(token));
    }


    /**
     * 验证令牌有效期，相差不足20分钟，自动刷新缓存
     * @param loginUser 登录用户
     */
    public void verifyToken(CurrentLoginUser loginUser) {
        long expireTime = loginUser.getExpireTime();
        long currentTime = System.currentTimeMillis();
        if (expireTime - currentTime <= Constants.UNIT_MILLIS_MINUTE_TEN) {
            refreshToken(loginUser);
        }
    }

    /**
     * 刷新令牌有效期
     * @param loginUser 登录用户
     */
    public void refreshToken(CurrentLoginUser loginUser) {
        loginUser.setLoginTime(System.currentTimeMillis());
        loginUser.setExpireTime(loginUser.getLoginTime() + expireTime * Constants.UNIT_MILLIS_SECOND);
        // 根据uuid将loginUser缓存
        String userKey = RedisKeyEnum.LOGIN_TOKEN_KEY.getKey(loginUser.getToken());
        redisCache.setCacheObject(userKey, loginUser, expireTime, TimeUnit.MINUTES);
    }


    /**
     * 创建令牌
     *
     * @param loginUser 用户信息
     * @return 令牌
     */
    public String createToken(CurrentLoginUser loginUser) {
        String token = UUID.randomUUID().toString();
        loginUser.setToken(token);
        setUserAgent(loginUser);
        refreshToken(loginUser);
        Map<String, Object> claims = new HashMap<>();
        claims.put(LOGIN_USER_KEY, token);
        return createToken(claims);
    }

    /**
     * 设置用户代理信息
     *
     * @param loginUser 登录信息
     */
    public void setUserAgent(CurrentLoginUser loginUser) {
        UserAgent userAgent = UserAgent.parseUserAgentString(getRequest().getHeader("User-Agent"));
        String ip = IpUtils.getIpAddr(getRequest());
        loginUser.setIpaddr(ip);
        loginUser.setLoginLocation(IpUtils.getRealAddressByIP(ip));
        loginUser.setBrowser(userAgent.getBrowser().getName());
        loginUser.setOs(userAgent.getOperatingSystem().getName());
    }


    /**
     * 获取 ServletRequestAttributes
     * @return
     */
    public ServletRequestAttributes getRequestAttributes() {
        RequestAttributes attributes = RequestContextHolder.getRequestAttributes();
        return (ServletRequestAttributes) attributes;
    }

    /**
     * 获取request
     */
    public HttpServletRequest getRequest() {
        return getRequestAttributes().getRequest();
    }

    /**
     * 从数据声明生成令牌
     *
     * @param claims 数据声明
     * @return 令牌
     */
    private String createToken(Map<String, Object> claims) {
        String token = Jwts.builder()
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, secret).compact();
        return token;
    }
}
