package com.hrsk.cloud.op.adapter.common.security;

import com.hrsk.cloud.op.app.system.SysPermissionSerivce;
import com.hrsk.cloud.op.domain.common.AuthErrorCodeEnum;
import com.hrsk.cloud.op.domain.system.SysUserStatusEnum;
import com.hrsk.cloud.op.infrastructure.repository.database.mysql.dao.SysUserDao;
import com.hrsk.cloud.op.infrastructure.repository.database.mysql.entity.SysUserDO;
import com.hrsk.cloud.op.infrastructure.repository.database.redis.RedisCache;
import com.hrsk.cloud.op.infrastructure.repository.database.redis.RedisKeyEnum;
import com.hrsk.pangu.tool.exception.SysException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

/**
 * @author: bianlanzhou
 * @create: 2024-10-14 17:52
 * @description: UserDetailsService
 **/
@Slf4j
@Component
public class UserDetailHelper implements UserDetailsService {
    @Resource
    private SysPermissionSerivce sysPermissionSerivce;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private RedisCache redisCache;

    @Value(value = "${security.user.password.maxRetryCount}")
    private int maxRetryCount;

    @Value(value = "${security.user.password.lockTime}")
    private int lockTime;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        SysUserDO sysUser = sysUserDao.selectUserByUserName(username);
        if (sysUser == null) {
            log.info("登录用户：{} 不存在.", username);
            throw new UsernameNotFoundException(AuthErrorCodeEnum.USER_NOT_FOUND.getMessage());
        }
        if (StringUtils.equals(sysUser.getStatus(), SysUserStatusEnum.DELETE.getCode())) {
            log.info("登录用户：{} 已被删除.", username);
            throw new SysException(AuthErrorCodeEnum.USER_HAS_DELETED.getCode(),AuthErrorCodeEnum.USER_HAS_DELETED.getMessage());
        } else if (StringUtils.equals(sysUser.getStatus(), SysUserStatusEnum.DEACTIVATE.getCode())) {
            log.info("登录用户：{} 已被停用.", username);
            throw new SysException(AuthErrorCodeEnum.USER_HAS_DEACTIVATED.getCode(),AuthErrorCodeEnum.USER_HAS_DEACTIVATED.getMessage());
        }
        validatePassword(sysUser.getPassword());
        return new CurrentLoginUser(sysUser, sysPermissionSerivce.getMenuPermission(sysUser) );
    }

    /**
     * 验证密码
     * @param password 密码
     */
    private void validatePassword(String password){
        Authentication usernamePasswordAuthenticationToken = AuthenticationContextHolder.getContext();
        String username = usernamePasswordAuthenticationToken.getName();
        String rawPassword = usernamePasswordAuthenticationToken.getCredentials().toString();
        Integer retryCount = redisCache.getCacheObject(RedisKeyEnum.PWD_ERR_CNT.getKey(username));
        if (retryCount == null) {
            retryCount = 0;
        }
        if (retryCount >= Integer.valueOf(maxRetryCount).intValue()) {
            throw new SysException(AuthErrorCodeEnum.USER_PASSWORD_RETRY_LIMIT_EXCEED.getCode(),AuthErrorCodeEnum.USER_PASSWORD_RETRY_LIMIT_EXCEED.getMessage());
        }
        if (!matches(rawPassword, password)) {
            retryCount = retryCount + 1;
            redisCache.setCacheObject(RedisKeyEnum.PWD_ERR_CNT.getKey(username), retryCount, lockTime, TimeUnit.MINUTES);
            throw new SysException(AuthErrorCodeEnum.PASSWORD_NOT_MATCH.getCode(),AuthErrorCodeEnum.PASSWORD_NOT_MATCH.getMessage());
        } else {
            redisCache.deleteObject(RedisKeyEnum.PWD_ERR_CNT.getKey(username));
        }
    }

    /**
     * 判断密码是否相同
     * @param rawPassword 真实密码
     * @param encodedPassword 加密后字符
     * @return 结果
     */
    public static boolean matches(String rawPassword, String encodedPassword) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.matches(rawPassword, encodedPassword);
    }

}
