package com.hrsk.cloud.op.adapter.controller.system;

import com.hrsk.cloud.op.adapter.common.security.AuthenticationContextHolder;
import com.hrsk.cloud.op.adapter.common.security.CurrentLoginUser;
import com.hrsk.cloud.op.adapter.common.security.HttpSecurityHelper;
import com.hrsk.cloud.op.adapter.common.security.TotpUtils;
import com.hrsk.cloud.op.client.api.AuthService;
import com.hrsk.cloud.op.client.dto.CaptchaDTO;
import com.hrsk.cloud.op.client.dto.CurrentLoginUserDTO;
import com.hrsk.cloud.op.client.dto.command.UserLoginCmd;
import com.hrsk.cloud.op.client.dto.command.UserOtpLoginCmd;
import com.hrsk.cloud.op.domain.common.AuthErrorCodeEnum;
import com.hrsk.pangu.dto.AntDSingleResponse;
import com.hrsk.pangu.dto.Response;
import com.hrsk.pangu.dto.SingleResponse;
import com.hrsk.pangu.tool.exception.BizException;
import com.hrsk.pangu.tool.exception.SysException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

/**
 * @author: bianlanzhou
 * @create: 2024-10-11 10:56
 * @description: 认证Controller
 **/
@Slf4j
@RestController
@RequestMapping("/auth")
public class AuthController {
    @Resource
    private AuthService authService;
    @Resource
    private AuthenticationManager authenticationManager;
    @Resource
    private HttpSecurityHelper httpSecurityHelper;

    /**
     * 图形验证码
     * @return 图形验证码
     * @throws IOException
     */
    @GetMapping("/captcha/image")
    public AntDSingleResponse<CaptchaDTO> captcha() throws IOException {
        SingleResponse<CaptchaDTO> response = authService.imageCaptcha();
        return  AntDSingleResponse.of(response);
    }

    /**
     * 登录
     * @param userLoginCmd 登录用户
     * @return token
     */
    @PostMapping("/login")
    public AntDSingleResponse<String> login(@RequestBody UserLoginCmd userLoginCmd){
        if(!authService.validateCaptcha(userLoginCmd.getCaptcha(), userLoginCmd.getUuid()).getData()){
            return AntDSingleResponse.buildFailure(AuthErrorCodeEnum.OTP_NOT_MATCH.getCode(),AuthErrorCodeEnum.OTP_NOT_MATCH.getMessage());
        }
        Authentication authentication = null;
        try {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(userLoginCmd.getUsername(), userLoginCmd.getPassword());
            AuthenticationContextHolder.setContext(authenticationToken);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        } catch (Exception e) {
            log.error("登录失败!",e);
            if (e instanceof BadCredentialsException) {
                return AntDSingleResponse.buildFailure(AuthErrorCodeEnum.PASSWORD_NOT_MATCH.getCode(),AuthErrorCodeEnum.PASSWORD_NOT_MATCH.getMessage());
            } else {
                if(e.getCause() instanceof SysException){
                    throw new SysException(((SysException) e.getCause()).getErrCode(),e.getMessage(),e);
                }else if(e.getCause() instanceof BizException){
                    throw new BizException(((SysException) e.getCause()).getErrCode(),e.getMessage(),e);
                }else{
                    throw e;
                }
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        CurrentLoginUser loginUser = (CurrentLoginUser) authentication.getPrincipal();
        return AntDSingleResponse.of(httpSecurityHelper.createToken(loginUser));
    }

    /**
     * otp登录
     * @param userOtpLoginCmd cmd
     * @return
     */
    @PostMapping("/otp-login")
    public Response otpLogin(@RequestBody UserOtpLoginCmd userOtpLoginCmd,HttpServletRequest request){
        CurrentLoginUser currentLoginUser = httpSecurityHelper.getCurrentLoginUser(request);
        if(currentLoginUser == null){
            return Response.buildFailure(AuthErrorCodeEnum.NOT_LOGIN.getCode(),AuthErrorCodeEnum.NOT_LOGIN.getMessage());
        }
        if(TotpUtils.verify(currentLoginUser.getOtpSecret(),userOtpLoginCmd.getCode())){
            currentLoginUser.setFinishTfa(true);
            httpSecurityHelper.refreshToken(currentLoginUser);
            return Response.buildSuccess();
        }
        return Response.buildFailure(AuthErrorCodeEnum.OTP_NOT_MATCH.getCode(),AuthErrorCodeEnum.OTP_NOT_MATCH.getMessage());
    }

    /**
     * 获取当前登录用户
     * @param request request
     * @return 当前登录用户
     */
    @GetMapping("/current-user")
    public AntDSingleResponse<CurrentLoginUserDTO> currentUser(HttpServletRequest request){
        CurrentLoginUser currentLoginUser = httpSecurityHelper.getCurrentLoginUser(request);
        return AntDSingleResponse.of(currentLoginUser.to());
    }
}
